/**
 * The scripthost message protocol and core utilities
 * @packageDocumentation
 */

/**
 * The response that is sent when an error occurred
 * @public
 */
export declare interface ErrorResponse extends GenericResponse<"error"> {
    message: string;
    /**
     * Optionally includes the global variables that were accessed during script evaluation.
     *
     * This property is omitted unless the error occurred during script evaluation and may
     * be omitted unless the script is evaluated with the `track` option.
     */
    vars?: Map<string, TrackedVariable>;
    /**
     * The value assigned to `this.refresh` during script evaluation.
     *
     * This property is omitted unless the error occurred during script evaluation and may
     * be omitted unless the script assigned `this.refresh`.
     */
    refresh?: ScriptValue;
}

/**
 * The message that is sent to request script evaluation
 * @public
 */
export declare interface EvaluateScriptRequest extends GenericMessage<"eval"> {
    /** The script that shall be evaluated */
    script: string;
    /**
     * Optionally identifies the instance in which the script shall be evaluated
     *
     * @remarks
     * This property shall be specified to use a specific `this` binding during script
     * evaluation.
     *
     * When not specified, a temporary `this` binding is used for each evaluation.
     */
    instanceId?: string;
    /**
     * Optionally specifies that the script is idempotent, i.e. is not allowed to have side-effects.
     * @remarks
     * An idempotent script cannot write to variables or call functions that may have side-effects.
     */
    idempotent?: boolean;
    /**
     * Optionally specifies that variables that are accessed during script evaluation
     * shall be tracked.
     */
    track?: boolean;
    /**
     * Optionally specifies variables that shall be made available to the script
     */
    vars?: Record<string, ScriptValue>;
}

/**
 * The response that is sent back after successful script evaluation
 * @public
 */
export declare interface EvaluateScriptResponse extends GenericResponse<"result"> {
    /** The result value */
    result: ScriptValue;
    /**
     * Optionally includes the global variables that were accessed during script evaluation.
     *
     * This property may be omitted unless the script is evaluated with the `track` option
     */
    vars?: Map<string, TrackedVariable>;
    /** The value assigned to `this.refresh` during script evaluation */
    refresh?: ScriptValue;
}

/**
 * The message that is sent from the underlying sandbox to invoke a function
 * exposed by the script host.
 * @public
 */
export declare interface FunctionCallRequest extends GenericMessage<"call"> {
    /** The function that shall be called */
    key: string;
    /** Function call arguments */
    args: ScriptValue[];
    /** Specifies whether the function is called from an idempotent script */
    idempotent: boolean;
    /** Unique identifier of the message that caused script evaluation */
    correlationId: string;
    /**
     * Optionally includes the global variables that were written during script evaluation,
     * before the current function call.
     *
     * The values of the provided map represents the version of the global scope that became
     * current after the corresponding variable was last written.
     *
     * This property may be omitted unless the script is evaluated with the `track` option
     */
    written?: Map<string, number>;
}

/**
 * The response that is sent back to the sandbox after a successful function call
 * @public
 */
export declare interface FunctionCallResponse extends GenericResponse<"return"> {
    /** The return value */
    result: ScriptValue;
}

/**
 * Generic message
 * @public
 */
export declare interface GenericMessage<T extends string = string> extends Partial<ScriptObject> {
    /** Message discriminator */
    type: T;
    /** Unique message identifier */
    messageId: string;
}

/**
 * Generic response
 * @public
 */
export declare interface GenericResponse<T extends string = string> extends GenericMessage<T> {
    /** Identifies the request message that this is a response to */
    inResponseTo: string;
}

/**
 * The message that is sent to initialize the underlying sandbox
 * @public
 */
export declare interface InitializeRequest extends GenericMessage<"init"> {
    /** Keys of exposed functions */
    funcs: Set<string>;
    /** Specifies whether global variables shall be read-only */
    readOnlyGlobals?: boolean;
}

/**
 * The message that is sent when the underlying sandbox is ready
 * @public
 */
export declare type InitializeResponse = GenericResponse<"ready">;

/**
 * Determines whether something is an {@link ErrorResponse}
 * @public
 */
export declare function isErrorResponse(thing: unknown): thing is ErrorResponse;

/**
 * Determines whether something is a {@link EvaluateScriptRequest}
 * @public
 */
export declare function isEvaluateScriptRequest(thing: unknown): thing is EvaluateScriptRequest;

/**
 * Determines whether something is a {@link EvaluateScriptResponse}
 * @public
 */
export declare function isEvaluateScriptResponse(thing: unknown): thing is EvaluateScriptResponse;

/**
 * Determines whether something is a {@link FunctionCallRequest}
 * @public
 */
export declare function isFunctionCallRequest(thing: unknown): thing is FunctionCallRequest;

/**
 * Determines whether something is a {@link FunctionCallResponse}
 * @public
 */
export declare function isFunctionCallResponse(thing: unknown): thing is FunctionCallResponse;

/**
 * Determines whether something is a {@link GenericMessage}
 * @param thing - The thing to test
 * @param type - Optionally specifies the message type to test
 * @public
 */
export declare function isGenericMessage<T extends string>(thing: unknown, type?: string): thing is GenericMessage<T>;

/**
 * Determines whether something is a {@link GenericResponse}
 * @param thing - The thing to test
 * @param type - Optionally specifies the message type to test
 * @public
 */
export declare function isGenericResponse<T extends string>(thing: unknown, type?: string): thing is GenericResponse<T>;

/**
 * Determines whether something is a {@link InitializeRequest}
 * @public
 */
export declare function isInitializeRequest(thing: unknown): thing is InitializeRequest;

/**
 * Determines whether something is a {@link InitializeResponse}
 * @public
 */
export declare function isInitializeResponse(thing: unknown): thing is InitializeResponse;

/**
 * Determines whether something is a {@link PingRequest}
 * @public
 */
export declare function isPingRequest(thing: unknown): thing is PingRequest;

/**
 * Determines whether something is a {@link PingResponse}
 * @public
 */
export declare function isPingResponse(thing: unknown): thing is PingResponse;

/**
 * Determines whether something is a {@link YieldRequest}
 * @public
 */
export declare function isYieldRequest(thing: unknown): thing is YieldRequest;

/**
 * Determines whether something is a {@link YieldResponse}
 * @public
 */
export declare function isYieldResponse(thing: unknown): thing is YieldResponse;

/**
 * The message that is sent to ping the other end of a messsage pipe
 * @public
 */
export declare type PingRequest = GenericMessage<"ping">;

/**
 * The message that is sent to in response to a ping
 * @public
 */
export declare type PingResponse = GenericResponse<"pong">;

/**
 * A script object
 * @public
 */
export declare interface ScriptObject extends Record<string, ScriptValue> {
}

/**
 * A sandbox in which scripts are run
 * @public
 */
export declare interface ScriptSandbox {
    /** Disposes the underlying sandbox */
    dispose(): void;
    /**
     * Sends a message to the sandbox
     * @param message - The message to be sent
     */
    post(message: ScriptValue): void;
    /**
     * Listens to messages from the sandbox
     * @param handler - The callback to be invoked whenever a message is received
     * @returns A callback that shall be called to stop receiving messages
     */
    listen(handler: (this: void, message: ScriptValue) => void): (this: void) => void;
}

/**
 * Alias for a function that construct {@link ScriptSandbox} instances
 * @public
 */
export declare type ScriptSandboxFactory = (this: void) => ScriptSandbox;

/**
 * A script value
 * @public
 */
export declare type ScriptValue = (void | boolean | null | undefined | number | BigInt | string | Date | RegExp | Blob | File | FileList | ArrayBuffer | ArrayBufferView | ImageBitmap | ImageData | Array<ScriptValue> | Map<ScriptValue, ScriptValue> | Set<ScriptValue> | ScriptObject);

/**
 * A tracked variable
 * @public
 */
export declare interface TrackedVariable extends Partial<ScriptObject> {
    /**
     * The version of the global scope when the variable was first read
     */
    read?: number;
    /**
     * The version of the global scope after the variable was last written
     */
    write?: number;
}

/**
 * The message that is sent from the underlying sandbox when script execution is yielded.
 * @public
 */
export declare interface YieldRequest extends GenericMessage<"yield"> {
    /** Unique identifier of the message that caused script evaluation */
    correlationId: string;
    /**
     * Optionally includes the global variables that were written during script evaluation,
     * before the current function call.
     *
     * The values of the provided map represents the version of the global scope that became
     * current after the corresponding variable was last written.
     *
     * This property may be omitted unless the script is evaluated with the `track` option.
     */
    written?: Map<string, number>;
}

/**
 * The response that is sent back to the sandbox to continue a yielded script
 * @public
 */
export declare type YieldResponse = GenericResponse<"continue">;

export { }
